@@ -456,6 +456,7 @@ private:
 #define UPX_F_VMLINUX_AMD64     27
 #define UPX_F_VMLINUX_ARMEL     28
 #define UPX_F_MACH_i386         29
+#define UPX_F_LINUX_ELF32_MIPSEL 30
 
 #define UPX_F_PLAIN_TEXT        127
 
@@ -466,6 +467,7 @@ private:
 #define UPX_F_LINUX_ELF32_ARMEB 133
 #define UPX_F_MACH_FAT          134
 #define UPX_F_VMLINUX_ARMEB     135
+#define UPX_F_VMLINUX_PPC32     136
 
 
 // compression methods
