@@ -159,7 +159,9 @@ void ElfLinker::init(const void *pdata_v, int plen)
         input = new upx_byte[inputlen + 1];
         unsigned new_len = u_len;
         int r = upx_decompress(pdata, c_len, input, &new_len, method, NULL);
-        if (r != 0 || new_len != u_len)
+        if (r == UPX_E_OUT_OF_MEMORY)
+            throwOutOfMemoryException();
+        if (r != UPX_E_OK || new_len != u_len)
             throwBadLoader();
     }
     else
@@ -801,6 +803,12 @@ void ElfLinkerPpc32::relocate1(const Relocation *rel, upx_byte *location,
         return super::relocate1(rel, location, value, type);
     type += 6;
 
+    if (strcmp(type, "ADDR32") == 0)
+    {
+        set_be32(location, get_be32(location) + value);
+        return;
+    }
+
     if (strncmp(type, "REL", 3) == 0)
     {
         value -= rel->section->offset + rel->offset;
