@@ -194,6 +194,8 @@ bool PackPs1::getBkupHeader(unsigned char *p, unsigned char *dst)
         unsigned sz_bh = SZ_IH_BKUP;
         int r = upx_decompress((const unsigned char *)&src->ih_bkup, src->len,
                                unc_bh, &sz_bh, M_NRV2E_8, NULL );
+        if (r == UPX_E_OUT_OF_MEMORY)
+            throwOutOfMemoryException();
         if (r != UPX_E_OK || sz_bh != SZ_IH_BKUP)
             throwInternalError("header decompression failed");
         unsigned ad = upx_adler32(unc_bh, SZ_IH_BKUP);
@@ -254,8 +256,9 @@ bool PackPs1::canPack()
     fi->readx(buf, sizeof(buf));
     checkAlreadyPacked(buf, sizeof(buf));
 
-    for (unsigned i = 0; i < sizeof(buf); i++)
+    for (size_t i = 0; i < sizeof(buf); i++)
         if (buf[i] != 0)
+        {
             if (!opt->force)
                 throwCantPack("unknown data in header (try --force)");
             else
@@ -265,6 +268,7 @@ bool PackPs1::canPack()
                 opt->backup = 1;
                 break;
             }
+        }
     if (!checkFileHeader())
         throwCantPack("unsupported header flags (try --force)");
     if (!opt->force && file_size < PS_MIN_SIZE)
@@ -318,7 +322,7 @@ void PackPs1::buildLoader(const Filter *)
             sa_tmp += overlap = ALIGN_UP((ph.overlap_overhead - sa_tmp), 4u);
     }
 
-    if (isCon || !isCon && M_IS_LZMA(ph.method))
+    if (isCon || M_IS_LZMA(ph.method))
         foundBss = findBssSection();
 
     if (M_IS_LZMA(ph.method) && !buildPart2)
@@ -572,11 +576,13 @@ void PackPs1::pack(OutputFile *fo)
     linker->defineSymbol("ldr_sz", M_IS_LZMA(ph.method) ? sz_lunc + 16 : (d_len-pad_code));
 
     if (foundBss)
+    {
         if (M_IS_LZMA(ph.method))
             linker->defineSymbol("wrkmem", bss_end - 160 - getDecompressorWrkmemSize()
                                            - (sz_lunc + 16));
         else
             linker->defineSymbol("wrkmem", bss_end - 16 - (d_len - pad_code));
+    }
 
 
     const unsigned entry = comp_data_start - e_len;
