@@ -2,8 +2,8 @@
 
    This file is part of the UPX executable compressor.
 
-   Copyright (C) 1996-2004 Markus Franz Xaver Johannes Oberhumer
-   Copyright (C) 1996-2004 Laszlo Molnar
+   Copyright (C) 1996-2007 Markus Franz Xaver Johannes Oberhumer
+   Copyright (C) 1996-2007 Laszlo Molnar
    All Rights Reserved.
 
    UPX and the UCL library are free software; you can redistribute them
@@ -62,7 +62,7 @@ bool PackUnix::canPack()
 #if defined(__unix__)
     // must be executable by owner
     if ((fi->st.st_mode & S_IXUSR) == 0)
-        throwCantPack("file not executable; try `chmod +x'");
+        throwCantPack("file not executable; try 'chmod +x'");
 #endif
     if (file_size < 4096)
         throwCantPack("file is too small");
@@ -125,8 +125,8 @@ int PackUnix::getStrategy(Filter &/*ft*/)
     // Called just before reading and compressing each block.
     // Might want to adjust blocksize, etc.
 
-    // If user specified the filter, then use it (-2==strategy).
-    // Else try the first two filters, and pick the better (2==strategy).
+    // If user specified the filter, then use it (-2==filter_strategy).
+    // Else try the first two filters, and pick the better (2==filter_strategy).
     return (opt->no_filter ? -3 : ((opt->filter > 0) ? -2 : 2));
 }
 
@@ -151,9 +151,9 @@ void PackUnix::pack2(OutputFile *fo, Filter &ft)
         // which assumes it has free choice on each call [block].
         // And if the choices aren't the same on each block,
         // then un-filtering will give incorrect results.
-        int strategy = getStrategy(ft);
+        int filter_strategy = getStrategy(ft);
         if (file_size > (off_t)blocksize)
-            strategy = -3;      // no filters
+            filter_strategy = -3;      // no filters
 
         int l = fi->readx(ibuf, UPX_MIN(blocksize, remaining));
         remaining -= l;
@@ -170,11 +170,13 @@ void PackUnix::pack2(OutputFile *fo, Filter &ft)
         // that is, AFTER filtering.  We want BEFORE filtering,
         // so that decompression checks the end-to-end checksum.
         unsigned const end_u_adler = upx_adler32(ibuf, ph.u_len, ph.u_adler);
-        compressWithFilters(&ft, OVERHEAD, strategy);
+        compressWithFilters(&ft, OVERHEAD, NULL_cconf, filter_strategy);
 
         if (ph.c_len < ph.u_len) {
+            const upx_bytep tbuf = NULL;
+            if (ft.id == 0) tbuf = ibuf;
             ph.overlap_overhead = OVERHEAD;
-            if (!testOverlappingDecompression(obuf, ph.overlap_overhead)) {
+            if (!testOverlappingDecompression(obuf, tbuf, ph.overlap_overhead)) {
                 // not in-place compressible
                 ph.c_len = ph.u_len;
             }
@@ -225,7 +227,7 @@ void PackUnix::pack2(OutputFile *fo, Filter &ft)
 void
 PackUnix::patchLoaderChecksum()
 {
-    unsigned char *const ptr = const_cast<unsigned char *>(getLoader());
+    unsigned char *const ptr = getLoader();
     l_info *const lp = &linfo;
     // checksum for loader; also some PackHeader info
     lp->l_magic = UPX_MAGIC_LE32;  // LE32 always
@@ -236,11 +238,10 @@ PackUnix::patchLoaderChecksum()
     lp->l_checksum = upx_adler32(ptr, lsize);
 }
 
-void PackUnix::pack3(OutputFile *fo, Filter &ft)
+void PackUnix::pack3(OutputFile *fo, Filter &/*ft*/)
 {
-    upx_byte const *p = getLoader();
+    upx_byte *p = getLoader();
     lsize = getLoaderSize();
-    patchFilter32(const_cast<upx_byte *>(p), lsize, &ft);
     updateLoader(fo);
     patchLoaderChecksum();
     fo->write(p, lsize);
@@ -305,21 +306,21 @@ void PackUnix::packExtent(
     unsigned &total_out,
     Filter *ft,
     OutputFile *fo,
-    unsigned hdr_ulen
+    unsigned hdr_u_len
 )
 {
     unsigned const init_u_adler = ph.u_adler;
     unsigned const init_c_adler = ph.c_adler;
     MemBuffer hdr_ibuf;
-    if (hdr_ulen) {
-        hdr_ibuf.alloc(hdr_ulen);
+    if (hdr_u_len) {
+        hdr_ibuf.alloc(hdr_u_len);
         fi->seek(0, SEEK_SET);
-        int l = fi->readx(hdr_ibuf, hdr_ulen);
+        int l = fi->readx(hdr_ibuf, hdr_u_len);
         (void)l;
     }
     fi->seek(x.offset, SEEK_SET);
     for (off_t rest = x.size; 0 != rest; ) {
-        int const strategy = getStrategy(*ft);
+        int const filter_strategy = getStrategy(*ft);
         int l = fi->readx(ibuf, UPX_MIN(rest, (off_t)blocksize));
         if (l == 0) {
             break;
@@ -346,17 +347,18 @@ void PackUnix::packExtent(
             ft->id = 0;
             ft->cto = 0;
 
-            compressWithFilters(ft, OVERHEAD, strategy,
-                NULL, 0, 0, 0, 0, // those 5 args are the defaults
-                hdr_ibuf, hdr_ulen);
+            compressWithFilters(ft, OVERHEAD, NULL_cconf, filter_strategy,
+                                0, 0, 0, hdr_ibuf, hdr_u_len);
         }
         else {
-            (void) compress(ibuf, obuf);    // ignore return value
+            (void) compress(ibuf, ph.u_len, obuf);    // ignore return value
         }
 
         if (ph.c_len < ph.u_len) {
+            const upx_bytep tbuf = NULL;
+            if (ft == NULL || ft->id == 0) tbuf = ibuf;
             ph.overlap_overhead = OVERHEAD;
-            if (!testOverlappingDecompression(obuf, ph.overlap_overhead)) {
+            if (!testOverlappingDecompression(obuf, tbuf, ph.overlap_overhead)) {
                 // not in-place compressible
                 ph.c_len = ph.u_len;
             }
@@ -371,31 +373,31 @@ void PackUnix::packExtent(
 
         // write block sizes
         b_info tmp;
-        if (hdr_ulen) {
-            unsigned hdr_clen;
+        if (hdr_u_len) {
+            unsigned hdr_c_len = 0;
             MemBuffer hdr_obuf;
-            unsigned result[16];
-            upx_compress_config_t conf;
-            memset(&conf, 0xff, sizeof(conf));
-            hdr_obuf.allocForCompression(hdr_ulen);
-            int r = upx_compress(hdr_ibuf, hdr_ulen, hdr_obuf, &hdr_clen, 0,
-                ph.method, 10, &conf, result);
-            (void)r;
-            ph.saved_u_adler = upx_adler32(hdr_ibuf, hdr_ulen, init_u_adler);
-            ph.saved_c_adler = upx_adler32(hdr_obuf, hdr_clen, init_c_adler);
+            hdr_obuf.allocForCompression(hdr_u_len);
+            int r = upx_compress(hdr_ibuf, hdr_u_len, hdr_obuf, &hdr_c_len, 0,
+                ph.method, 10, NULL, NULL);
+            if (r != UPX_E_OK)
+                throwInternalError("header compression failed");
+            if (hdr_c_len >= hdr_u_len)
+                throwInternalError("header compression size increase");
+            ph.saved_u_adler = upx_adler32(hdr_ibuf, hdr_u_len, init_u_adler);
+            ph.saved_c_adler = upx_adler32(hdr_obuf, hdr_c_len, init_c_adler);
             ph.u_adler = upx_adler32(ibuf, ph.u_len, ph.saved_u_adler);
             ph.c_adler = upx_adler32(obuf, ph.c_len, ph.saved_c_adler);
             end_u_adler = ph.u_adler;
             memset(&tmp, 0, sizeof(tmp));
-            set_native32(&tmp.sz_unc, hdr_ulen);
-            set_native32(&tmp.sz_cpr, hdr_clen);
+            set_native32(&tmp.sz_unc, hdr_u_len);
+            set_native32(&tmp.sz_cpr, hdr_c_len);
             tmp.b_method = (unsigned char) ph.method;
             fo->write(&tmp, sizeof(tmp));
             b_len += sizeof(b_info);
-            fo->write(hdr_obuf, hdr_clen);
-            total_out += hdr_clen;
-            total_in  += hdr_ulen;
-            hdr_ulen = 0;  // compress hdr one time only
+            fo->write(hdr_obuf, hdr_c_len);
+            total_out += hdr_c_len;
+            total_in  += hdr_u_len;
+            hdr_u_len = 0;  // compress hdr one time only
         }
         memset(&tmp, 0, sizeof(tmp));
         set_native32(&tmp.sz_unc, ph.u_len);
@@ -501,7 +503,7 @@ int PackUnix::canUnpack()
 
     fi->seek(-bufsize, SEEK_END);
     fi->readx(buf, bufsize);
-    if (!getPackHeader(buf, bufsize))
+    if (!getPackHeader(buf, bufsize, true))  // allow incompressible extents
         return false;
 
     int l = ph.buf_offset + ph.getPackHeaderSize();
