@@ -46,6 +46,8 @@ static const
 #include "stub/arm-linux.kernel.vmlinux.h"
 static const
 #include "stub/armeb-linux.kernel.vmlinux.h"
+static const
+#include "stub/powerpc-linux.kernel.vmlinux.h"
 
 
 /*************************************************************************
@@ -203,6 +205,7 @@ bool PackVmlinuxBase<T>::canPack()
     return 0 < n_ptload;
 }
 
+#include "p_elf.h"
 
 template <class T>
 void PackVmlinuxBase<T>::pack(OutputFile *fo)
@@ -267,13 +270,224 @@ void PackVmlinuxBase<T>::pack(OutputFile *fo)
     upx_compress_config_t cconf; cconf.reset();
     // limit stack size needed for runtime decompression
     cconf.conf_lzma.max_num_probs = 1846 + (768 << 4); // ushort: ~28KB stack
-    compressWithFilters(&ft, 512, &cconf, getStrategy(ft));
+
+    unsigned ppc32_extra = 0;
+    if (Ehdr::EM_PPC==my_e_machine) {
+        // output layout:
+        //      .long UPX_MAGIC_LE32
+        //      .long L20 - L10
+        // L10:
+        //      b_info for Ehdr; compressed Ehdr; .balign 4  // one block only
+        //      b_info for LOAD; compressed LOAD; .balign 4  // possibly many blocks
+        //          // This allows per-block filters!
+        // L20:
+        //      b f_decompress
+        // +4:  f_unfilter(char *buf, unsigned len, unsigned cto8, unsigned ftid)
+        //          // Code for multiple filters can "daisy chain" on ftid.
+        //      f_decompress(char const *src, unsigned  src_len,
+        //                   char       *dst, unsigned *dst_len, int method)
+        unsigned tmp;
+        tmp = UPX_MAGIC_LE32; fo->write(&tmp, sizeof(tmp)); fo_off += sizeof(tmp);
+        tmp = 0;              fo->write(&tmp, sizeof(tmp)); fo_off += sizeof(tmp);
+        ppc32_extra += 2*sizeof(tmp);
+        unsigned const len_unc = sizeof(ehdri) + sizeof(Phdr) * ehdri.e_phnum;
+        MemBuffer unc_hdr(len_unc);
+        MemBuffer cpr_hdr; cpr_hdr.allocForCompression(len_unc);
+        memcpy(&unc_hdr[0],             &ehdri, sizeof(ehdri));
+        memcpy(&unc_hdr[sizeof(ehdri)],  phdri, sizeof(Phdr) * ehdri.e_phnum);
+        unsigned len_cpr = 0;
+        int const r = upx_compress(unc_hdr, len_unc, cpr_hdr, &len_cpr,
+            NULL, ph.method, 10, NULL, NULL );
+        if (UPX_E_OK!=r || len_unc<=len_cpr)  // FIXME: allow no compression
+            throwInternalError("Ehdr compression failed");
+
+        struct b_info { // 12-byte header before each compressed block
+            unsigned sz_unc;  // uncompressed_size
+            unsigned sz_cpr;  //   compressed_size
+            unsigned char b_method;  // compression algorithm
+            unsigned char b_ftid;  // filter id
+            unsigned char b_cto8;  // filter parameter
+            unsigned char b_unused;  // FIXME: !=0  for partial-block unfilter
+            // unsigned f_offset, f_len;  // only if    partial-block unfilter
+        }
+        __attribute_packed;
+
+        struct b_info hdr_info;
+        set_be32(&hdr_info.sz_unc, len_unc);
+        set_be32(&hdr_info.sz_cpr, len_cpr);
+        hdr_info.b_method = ph.method;
+        hdr_info.b_ftid = 0;
+        hdr_info.b_cto8 = 0;
+        hdr_info.b_unused = 0;
+        fo->write(&hdr_info, sizeof(hdr_info)); fo_off += sizeof(hdr_info);
+        unsigned const frag = (3& (0u-len_cpr));
+        ppc32_extra += sizeof(hdr_info) + len_cpr + frag;
+        fo_off += len_cpr + frag;
+        fo->write(cpr_hdr, len_cpr + frag);
+
+        // Partial filter: .text and following contiguous SHF_EXECINSTR
+        upx_bytep f_ptr = ibuf;
+        unsigned  f_len = 0;
+        Shdr const *shdr = 1+ shdri;  // skip empty shdr[0]
+        if (ft.buf_len==0  // not specified yet  FIXME: was set near construction
+        && (Shdr::SHF_ALLOC     & shdr->sh_flags)
+        && (Shdr::SHF_EXECINSTR & shdr->sh_flags)) {
+            // shdr[1] is instructions (probably .text)
+            f_ptr = ibuf + (unsigned) (shdr->sh_offset - phdri[0].p_offset);
+            f_len = shdr->sh_size;
+            ++shdr;
+            for (int j= ehdri.e_shnum - 2; --j>=0; ++shdr) {
+                unsigned prev_end = shdr[-1].sh_size + shdr[-1].sh_offset;
+                prev_end += ~(0u-shdr[0].sh_addralign) & (0u-prev_end);  // align_up
+                if ((Shdr::SHF_ALLOC     & shdr->sh_flags)
+                &&  (Shdr::SHF_EXECINSTR & shdr->sh_flags)
+                &&  shdr[0].sh_offset==prev_end) {
+                    f_len += shdr->sh_size;
+                }
+                else {
+                    break;
+                }
+            }
+        }
+        else { // ft.buf_len already specified, or Shdr[1] not instructions
+            f_ptr = ibuf;
+            f_len = ph.u_len;
+        }
+        compressWithFilters(ibuf, ph.u_len, obuf,
+            f_ptr, f_len,  // filter range
+            NULL, 0,  // hdr_ptr, hdr_len
+            &ft, 512, &cconf, getStrategy(ft));
+
+        set_be32(&hdr_info.sz_unc, ph.u_len);
+        set_be32(&hdr_info.sz_cpr, ph.c_len);
+        hdr_info.b_ftid = ft.id;
+        hdr_info.b_cto8 = ft.cto;
+        if (ph.u_len!=f_len) {
+            hdr_info.b_unused = 1;  // flag for partial filter
+        }
+        fo->write(&hdr_info, sizeof(hdr_info)); fo_off += sizeof(hdr_info);
+        ppc32_extra += sizeof(hdr_info);
+        if (ph.u_len!=f_len) {
+            set_be32(&hdr_info.sz_unc, f_ptr - (upx_bytep) ibuf);
+            set_be32(&hdr_info.sz_cpr, f_len);
+            fo->write(&hdr_info, 2*sizeof(unsigned)); fo_off += 2*sizeof(unsigned);
+            ppc32_extra += 2*sizeof(unsigned);
+        }
+
+#if 0  /*{ Documentation: changes to arch/powerpc/boot/main.c */
+struct b_info {
+	unsigned sz_unc;
+	unsigned sz_cpr;
+	unsigned char b_method;
+	unsigned char b_ftid;
+	unsigned char b_cto8;
+	unsigned char b_unused;
+};
+typedef int (*upx_f_unc)(  /* uncompress */
+	unsigned char const *src, unsigned  src_len,
+	unsigned char       *dst, unsigned *dst_len,
+	int method
+);
+typedef int (*upx_f_unf)(  /* unfilter */
+	unsigned char *buf,
+	unsigned len,
+	unsigned cto8,
+	unsigned ftid
+);
+
+unsigned char const *upx_expand(  // return updated src [aligned, too]
+	unsigned char const *src,
+	unsigned char       *dst,
+	unsigned *const dst_len,
+	upx_f_unc const f_unc
+)
+{
+	if (0==*dst_len) {
+		*dst_len = ((struct b_info const *)src)->sz_unc;
+	}
+	upx_f_unf const f_unf = (upx_f_unf)(sizeof(int) + (char *)f_unc);
+	unsigned total_len = 0;
+	unsigned need = *dst_len;
+	while (0 < need) {
+		struct b_info const *const b_hdr = (struct b_info const *)src;
+		src = (unsigned char const *)(1+ b_hdr);
+		unsigned blk_len = b_hdr->sz_unc;
+		int const rv = (*f_unc)(src, b_hdr->sz_cpr, dst, &blk_len, b_hdr->b_method);
+		if (0!=rv) {
+			printf("decompress error %d\n\r", rv);
+			exit();
+		}
+		if (b_hdr->b_ftid) {
+			(*f_unf)(dst, b_hdr->sz_unc, b_hdr->b_cto8, b_hdr->b_ftid);
+		}
+		src += b_hdr->sz_cpr;
+		src += (3& -(int)src);  // 4-byte align
+		dst += b_hdr->sz_unc;
+		total_len += b_hdr->sz_unc;
+		need -= b_hdr->sz_unc;
+	}
+	*dst_len = total_len;
+	return src;
+}
+
+void prep_kernel(unsigned long a1, unsigned long a2)
+{
+	upx_f_unc f_unc = (upx_f_unc)0;
+
+	vmlinuz.addr = (unsigned long)_vmlinux_start;
+	vmlinuz.size = (unsigned long)(_vmlinux_end - _vmlinux_start);
+
+	if (0x55505821 == *(unsigned *)vmlinuz.addr) {
+		/* was compressed by upx */
+		/* Uncompress elfheader */
+		unsigned const *const fwa = (unsigned *)vmlinuz.addr;
+		f_unc = (upx_f_unc)((char *)&fwa[2] + fwa[1]);
+		unsigned dst_len = 0;
+		unsigned char const *src = (unsigned char const *)&fwa[2];
+		src = upx_expand(src, (unsigned char *)elfheader, &dst_len, f_unc);
+		unsigned const src_len = src - (unsigned char const *)&fwa[0];
+		vmlinuz.addr += src_len;
+		vmlinuz.size -= src_len;
+	} else {
+		memcpy(elfheader, (const void *)vmlinuz.addr,
+		       sizeof(elfheader));
+	}
+=====
+	/* Eventually gunzip the kernel */
+	if (f_unc) {
+		/* was compressed by upx */
+		printf("upx_expand (0x%lx <- 0x%lx:0x%0lx)...",
+		       vmlinux.addr, vmlinuz.addr, vmlinuz.addr+vmlinuz.size);
+		unsigned dst_len = vmlinux.size - elfoffset;
+		upx_expand((unsigned char const *)vmlinuz.addr,
+		           (unsigned char       *)vmlinux.addr, &dst_len, f_unc);
+		printf("done 0x%lx bytes\n\r", len);
+	} else {
+		memmove((void *)vmlinux.addr,(void *)vmlinuz.addr,
+			vmlinuz.size);
+		/* Skip over the ELF header */
+#ifdef DEBUG
+		printf("... skipping 0x%lx bytes of ELF header\n\r",
+				elfoffset);
+#endif
+		vmlinux.addr += elfoffset;
+	}
+=====
+}
+#endif  /*}*/
+
+    }
+    else {
+        compressWithFilters(&ft, 512, &cconf, getStrategy(ft));
+    }
     unsigned const txt_c_len = ph.c_len;
 
     const unsigned lsize = getLoaderSize();
 
     defineDecompressorSymbols();
-    defineFilterSymbols(&ft);
+    if (ft.id!=0) {
+        defineFilterSymbols(&ft);
+    }
     relocateLoader();
 
     MemBuffer loader(lsize);
@@ -287,13 +501,13 @@ void PackVmlinuxBase<T>::pack(OutputFile *fo)
     shdro[1].sh_name = ptr_diff(p, shstrtab);
     shdro[1].sh_type = Shdr::SHT_PROGBITS;
     shdro[1].sh_flags = Shdr::SHF_ALLOC | Shdr::SHF_EXECINSTR;
-    shdro[1].sh_offset = fo_off;
-    shdro[1].sh_size = txt_c_len + lsize;  // plus more ...
+    shdro[1].sh_offset = fo_off - ppc32_extra;
+    shdro[1].sh_size = ppc32_extra + txt_c_len + lsize;  // plus more ...
     shdro[1].sh_addralign = 1;  // default
 
     fo_off += write_vmlinux_head(fo, &shdro[1]);
     fo->write(obuf, txt_c_len); fo_off += txt_c_len;
-    unsigned const a = (shdro[1].sh_addralign -1) & (0u-txt_c_len);
+    unsigned const a = (shdro[1].sh_addralign -1) & (0u-(ppc32_extra + txt_c_len));
     if (0!=a) { // align
         fo_off += a;
         shdro[1].sh_size += a;
@@ -361,7 +575,7 @@ void PackVmlinuxBase<T>::pack(OutputFile *fo)
     shdro[5].sh_name = ptr_diff(p, shstrtab);
     shdro[5].sh_type = Shdr::SHT_SYMTAB;
     shdro[5].sh_offset = fo_off;
-    shdro[5].sh_size = 5*sizeof(Sym);
+    shdro[5].sh_size = ((Ehdr::EM_PPC==my_e_machine) + 5)*sizeof(Sym);
     //shdro[5].sh_flags = Shdr::SHF_INFO_LINK;
     shdro[5].sh_link = 6;  // to .strtab for symbols
     shdro[5].sh_info = 1+3;  // number of non-global symbols [binutils/bfd/elf.c]
@@ -388,26 +602,40 @@ void PackVmlinuxBase<T>::pack(OutputFile *fo)
     Sym unc_ker;
     unc_ker.st_name = 1;  // 1 byte into strtab
     unc_ker.st_value = 0;
-    unc_ker.st_size = txt_c_len;
+    unc_ker.st_size = ppc32_extra + txt_c_len;
     unc_ker.st_info = unc_ker.make_st_info(Sym::STB_GLOBAL, Sym::STT_FUNC);
     unc_ker.st_other = Sym::STV_DEFAULT;
     unc_ker.st_shndx = 1;  // .text
     fo->write(&unc_ker, sizeof(unc_ker)); fo_off += sizeof(unc_ker);
 
     unsigned const lablen = strlen(my_boot_label);
+    if (Ehdr::EM_PPC==my_e_machine) {
+        unc_ker.st_name += 1+ lablen;
+        unc_ker.st_value = unc_ker.st_size;
+        unc_ker.st_size = 0;
+        fo->write(&unc_ker, sizeof(unc_ker)); fo_off += sizeof(unc_ker);
+    }
     while (0!=*p++) ;
     shdro[6].sh_name = ptr_diff(p, shstrtab);
     shdro[6].sh_type = Shdr::SHT_STRTAB;
     shdro[6].sh_offset = fo_off;
-    shdro[6].sh_size = 2+ lablen;  // '\0' before and after
+    shdro[6].sh_size = 2+ lablen + (Ehdr::EM_PPC==my_e_machine)*(1+ 12);  // '\0' before and after
     shdro[6].sh_addralign = 1;
     fo->seek(1, SEEK_CUR);  // the '\0' before
     fo->write(my_boot_label, 1+ lablen);  // include the '\0' terminator
+    if (Ehdr::EM_PPC==my_e_machine) {
+        fo->write("_vmlinux_end", 1+ 12); fo_off += 1+ 12;
+    }
     fo_off += 2+ lablen;
 
     fo->seek(0, SEEK_SET);
     fo->write(&ehdro, sizeof(ehdro));
     fo->write(&shdro, sizeof(shdro));
+    if (Ehdr::EM_PPC==my_e_machine) {
+        fo->seek(sizeof(unsigned), SEEK_CUR);
+        set_be32(&ppc32_extra, ppc32_extra - 2*sizeof(unsigned) + txt_c_len);
+        fo->write(&ppc32_extra, sizeof(ppc32_extra));
+    }
 
     if (!checkFinalCompressionRatio(fo))
         throwNotCompressible();
@@ -559,6 +787,12 @@ const int *PackVmlinuxARMEB::getCompressionMethods(int method, int level) const
     return Packer::getDefaultCompressionMethods_8(method, level);
 }
 
+const int *PackVmlinuxPPC32::getCompressionMethods(int method, int level) const
+{
+    // No real dependency on LE32.
+    return Packer::getDefaultCompressionMethods_le32(method, level);
+}
+
 
 const int *PackVmlinuxARMEL::getFilters() const
 {
@@ -572,6 +806,12 @@ const int *PackVmlinuxARMEB::getFilters() const
     return f51;
 }
 
+const int *PackVmlinuxPPC32::getFilters() const
+{
+    static const int fd0[] = { 0xd0, FT_END };
+    return fd0;
+}
+
 //
 // Examples as of 2004-07-16 [readelf --segments vmlinux  # before fiddling]:
 //
@@ -669,6 +909,11 @@ bool PackVmlinuxARMEB::is_valid_e_entry(Addr e_entry)
     return 0xc0008000==e_entry;
 }
 
+bool PackVmlinuxPPC32::is_valid_e_entry(Addr e_entry)
+{
+    return 0xc0000000==e_entry;
+}
+
 Linker* PackVmlinuxARMEL::newLinker() const
 {
     return new ElfLinkerArmLE;
@@ -679,6 +924,11 @@ Linker* PackVmlinuxARMEB::newLinker() const
     return new ElfLinkerArmBE;
 }
 
+Linker* PackVmlinuxPPC32::newLinker() const
+{
+    return new ElfLinkerPpc32;
+}
+
 
 
 void PackVmlinuxARMEL::buildLoader(const Filter *ft)
@@ -725,6 +975,28 @@ void PackVmlinuxARMEB::buildLoader(const Filter *ft)
     addLoader("IDENTSTR,UPX1HEAD", NULL);
 }
 
+void PackVmlinuxPPC32::buildLoader(const Filter *ft)
+{
+    // prepare loader
+    initLoader(stub_powerpc_linux_kernel_vmlinux, sizeof(stub_powerpc_linux_kernel_vmlinux));
+    addLoader("LINUX000", NULL);
+    if (ft->id) {
+        assert(ft->calls > 0);
+        addLoader("LINUX010", NULL);
+    }
+    addLoader("LINUX020", NULL);
+    if (ft->id) {
+        addFilter32(ft->id);
+    }
+    addLoader("LINUX030", NULL);
+         if (ph.method == M_NRV2E_LE32) addLoader("NRV2E", NULL);
+    else if (ph.method == M_NRV2B_LE32) addLoader("NRV2B", NULL);
+    else if (ph.method == M_NRV2D_LE32) addLoader("NRV2D", NULL);
+    else if (M_IS_LZMA(ph.method))   addLoader("LZMA_ELF00,LZMA_DEC10,LZMA_DEC30", NULL);
+    else throwBadLoader();
+    addLoader("IDENTSTR,UPX1HEAD", NULL);
+}
+
 
 static const
 #include "stub/i386-linux.kernel.vmlinux-head.h"
@@ -734,6 +1006,8 @@ static const
 #include "stub/arm-linux.kernel.vmlinux-head.h"
 static const
 #include "stub/armeb-linux.kernel.vmlinux-head.h"
+static const
+#include "stub/powerpc-linux.kernel.vmlinux-head.h"
 
 unsigned PackVmlinuxI386::write_vmlinux_head(
     OutputFile *const fo,
@@ -783,6 +1057,14 @@ void PackVmlinuxARMEB::defineDecompressorSymbols()
     linker->defineSymbol("METHOD", ph.method);
 }
 
+void PackVmlinuxPPC32::defineDecompressorSymbols()
+{
+    super::defineDecompressorSymbols();
+    // linker->defineSymbol(  "COMPRESSED_LENGTH", ph.c_len);
+    // linker->defineSymbol("UNCOMPRESSED_LENGTH", ph.u_len);
+    // linker->defineSymbol("METHOD", ph.method);
+}
+
 void PackVmlinuxI386::defineDecompressorSymbols()
 {
     super::defineDecompressorSymbols();
@@ -844,6 +1126,14 @@ unsigned PackVmlinuxARMEB::write_vmlinux_head(
     return sizeof(stub_armeb_linux_kernel_vmlinux_head);
 }
 
+unsigned PackVmlinuxPPC32::write_vmlinux_head(
+    OutputFile * /*const fo*/,
+    Shdr * /*const stxt*/
+)
+{
+    return 0;
+}
+
 
 bool PackVmlinuxARMEL::has_valid_vmlinux_head()
 {
@@ -873,6 +1163,20 @@ bool PackVmlinuxARMEB::has_valid_vmlinux_head()
     return false;
 }
 
+bool PackVmlinuxPPC32::has_valid_vmlinux_head()
+{
+    U32 buf[2];
+    fi->seek(p_text->sh_offset + sizeof(stub_powerpc_linux_kernel_vmlinux_head) -8, SEEK_SET);
+    fi->readx(buf, sizeof(buf));
+    //unsigned const word0 = buf[0];
+    unsigned const word1 = buf[1];
+    if (0xeb==(word1>>24)
+    &&  (0x00ffffff& word1)==(0u - 1 + ((3+ ph.c_len)>>2))) {
+        return true;
+    }
+    return false;
+}
+
 bool PackVmlinuxI386::has_valid_vmlinux_head()
 {
     unsigned char buf[5];
