@@ -592,6 +592,10 @@ void PackW32Pe::buildLoader(const Filter *ft)
 
 void PackW32Pe::pack(OutputFile *fo)
 {
+    // FIXME: we need to think about better support for --exact
+    if (opt->exact)
+        throwCantPackExact();
+
     const unsigned objs = ih.objects;
     isection = new pe_section_t[objs];
     fi->seek(pe_offset+sizeof(ih),SEEK_SET);
@@ -629,10 +633,12 @@ void PackW32Pe::pack(OutputFile *fo)
     else if (opt->win32_pe.strip_relocs < 0)
         opt->win32_pe.strip_relocs = (ih.imagebase >= 0x400000);
     if (opt->win32_pe.strip_relocs)
+    {
         if (ih.imagebase < 0x400000)
             throwCantPack("--strip-relocs is not allowed when imagebase < 0x400000");
         else
             ih.flags |= RELOCS_STRIPPED;
+    }
 
     if (memcmp(isection[0].name,"UPX",3) == 0)
         throwAlreadyPackedByUPX();
@@ -1021,6 +1027,8 @@ void PackW32Pe::pack(OutputFile *fo)
     oh.codebase = osection[1].vaddr;
     // oh.headersize = osection[0].rawdataptr;
     oh.headersize = rvamin;
+    if (rvamin < osection[0].rawdataptr)
+        throwCantPack("object alignment too small");
 
     if (opt->win32_pe.strip_relocs && !isdll)
         oh.flags |= RELOCS_STRIPPED;
