@@ -196,6 +196,8 @@ Packer* PackMaster::visitAllPackers(visit_func_t func, InputFile *f, const optio
         return p;
     if ((p = func(new PackVmlinuxARMEB(f), user)) != NULL)
         return p;
+    if ((p = func(new PackVmlinuxPPC32(f), user)) != NULL)
+        return p;
     if ((p = func(new PackVmlinuxAMD64(f), user)) != NULL)
         return p;
     if ((p = func(new PackVmlinuxI386(f), user)) != NULL)
@@ -241,6 +243,8 @@ Packer* PackMaster::visitAllPackers(visit_func_t func, InputFile *f, const optio
             return p;
         if ((p = func(new PackLinuxElf32ppc(f), user)) != NULL)
             return p;
+        if ((p = func(new PackLinuxElf32mipsel(f), user)) != NULL)
+            return p;
         if ((p = func(new PackLinuxI386sh(f), user)) != NULL)
             return p;
     }
